from parser import Parser
from possibilities import apply_suggestion, BEST_POSSIBILITY_CACHE
from strategy import find_possibilities
from tests.parser import ParserWrapper


def validate(exp, result):
    """
    Validate that exp =>* result.
    """
    parser = ParserWrapper(Parser)
    exp = parser.run([exp])
    result = parser.run([result])

    def traverse_preorder(node, result):
        if node.equals(result):
            return True

        node_expr = str(node)

        possibilities = find_possibilities(node)

        # If a best possibility chain of rewrite steps is discovered before,
        # use the corresponding possibility index of that chain instead.
        if node_expr in BEST_POSSIBILITY_CACHE:
            p = BEST_POSSIBILITY_CACHE[node_expr]

            # If there is no possible rewrite step, bail out
            if p is None:
                return False

            possibilities = possibilities[p]

        for p in possibilities:
            # Clone the root node because it will be used in multiple
            # substitutions
            child = apply_suggestion(node.clone(), p)

            if traverse_preorder(child, result):
                return True

        return False

    return traverse_preorder(exp, result)
